#include "DeviceDht.h"

void DeviceDht::deviceSetup() {
  pinMode(pin, INPUT);
  dht.begin();
}

void DeviceDht::deviceRegister() {
  Homie.registerNode(humidityNode);
  Homie.registerNode(temperatureNode);
  Homie.registerNode(heatindexNode);
}

// compare float values
bool isEqual(float a, float b, float epsilon=0.001) {
  return fabs(a - b) <= epsilon * fabs(a);
}

void DeviceDht::deviceLoop() {
  if (millis() - lastSentDHT >= INTERVAL_DHT * 1000UL || lastSentDHT == 0) {
    float previousHumidity = humidity;
    float previousTemperature = temperature;
    float previousHeatindex = heatindex;
    humidity = dht.readHumidity();        // read humidity as a percent
    temperature = dht.readTemperature();  // read temperature as Celsius
    heatindex = dht.computeHeatIndex(temperature, humidity, false);

    // check if any reads failed and exit early
    if (isnan(humidity) || isnan(temperature)) {
      Serial.println("Failed to read from DHT sensor!");
      return;
    }

    if (!isEqual(humidity, previousHumidity)) {
      Serial.print("humidity: ");
      Serial.println(humidity);
      if (!Homie.setNodeProperty(humidityNode, "value", String(humidity), true)) {
        Serial.println("Sending failed");
      }
    } else {
      Serial.println("humidity unchanged");
    }

    if (!isEqual(temperature, previousTemperature)) {
      Serial.print("temperature: ");
      Serial.println(temperature);
      if (!Homie.setNodeProperty(temperatureNode, "value", String(temperature), true)) {
        Serial.println("Sending failed");
      }
    } else {
      Serial.println("temperature unchanged");
    }

    if (!isEqual(heatindex, previousHeatindex)) {
      Serial.print("heatindex: ");
      Serial.println(heatindex);
      if (!Homie.setNodeProperty(heatindexNode, "value", String(heatindex), true)) {
        Serial.println("Sending failed");
      }
    } else {
      Serial.println("heatindex unchanged");
    }

    lastSentDHT = millis();
  }
}
